/* 
 * hashinterface.c
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pthread.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <openssl/md5.h>
#include <openssl/sha.h>
#include <openssl/des.h>
#include <openssl/pem.h>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/rsa.h>
#include <openssl/hmac.h>
#include <openssl/ripemd.h>
#include <openssl/aes.h>
#include "err.h"
#include "hashinterface.h"
#include "threads.h"



/* Global variables */
char temp_username[HASHFILE_MAX_LINE_LENGTH];	// temporary username
char temp_salt[HASHFILE_MAX_LINE_LENGTH];	// temporary salt
char temp_salt2[HASHFILE_MAX_LINE_LENGTH];	// temporary salt2
char temp_hash[HASHFILE_MAX_LINE_LENGTH];	// temporary hash


/* Function prototypes */
void hash_proto_md5(const char *plaintext, char *hash, int len);
void hash_proto_md5_hex(const char *hash, char *hashhex);
void hash_proto_sha1(const char *plaintext, char *hash, int len);
void hash_proto_sha1_hex(const char *hash, char *hashhex);
void hash_proto_sha256(const char *plaintext, char *hash, int len);
void hash_proto_sha256_hex(const char *hash, char *hashhex);
void hash_proto_sha512(const char *plaintext, char *hash, int len);
void hash_proto_sha512_hex(const char *hash, char *hashhex);
void hash_proto_fcrypt(const char *password, const char *salt, char *ret);
void hash_proto_add_username(const char *username);
void hash_proto_add_hash(const char *hash, int len);
void hash_proto_add_salt(const char *salt);
void hash_proto_add_salt2(const char *salt2);
hash_stat add_hash_list(char *username, char *hash, char *salt, char *salt2);
hash_stat add_cracked_list(char *username, char *hash, char *salt, char *salt2);
hash_stat del_hash_list(struct hash_list_s *node);
hash_stat del_cracked_list(struct hash_list_s *node);
void print_hash_list(void);
void print_cracked_list(void);
void print_cracked_list_to_file(char *filename);
int get_cracked_num(void);
int get_hashes_num(void);
void cleanup_lists(void);
void markov_attack_init(void);
void markov_print_statfiles(void);
hash_stat markov_load_statfile(char *statname);


static char privkey[8192];



/* MD5() proto function */
void hash_proto_md5(const char *plaintext, char * __restrict hash, int len)
{
    MD5_CTX ctx;
    MD5_Init(&ctx);
    MD5_Update(&ctx, plaintext, len);
    MD5_Final((unsigned char *)hash,&ctx);
}



/* MD5 digest -> hex str */
void hash_proto_md5_hex(const char *hash, char *hashhex)
{
    unsigned char charset[17]="0123456789abcdef";

    /* I love chaos :) */
    *(hashhex+1) = charset[(unsigned char)*(hash)&15];
    *(hashhex+0) = charset[(unsigned char)*(hash)>>4];
    *(hashhex+3) = charset[(unsigned char)*(hash+1)&15];
    *(hashhex+2) = charset[(unsigned char)*(hash+1)>>4];
    *(hashhex+5) = charset[(unsigned char)*(hash+2)&15];
    *(hashhex+4) = charset[(unsigned char)*(hash+2)>>4];
    *(hashhex+7) = charset[(unsigned char)*(hash+3)&15];
    *(hashhex+6) = charset[(unsigned char)*(hash+3)>>4];
    *(hashhex+9) = charset[(unsigned char)*(hash+4)&15];
    *(hashhex+8) = charset[(unsigned char)*(hash+4)>>4];
    *(hashhex+11) = charset[(unsigned char)*(hash+5)&15];
    *(hashhex+10) = charset[(unsigned char)*(hash+5)>>4];
    *(hashhex+13) = charset[(unsigned char)*(hash+6)&15];
    *(hashhex+12) = charset[(unsigned char)*(hash+6)>>4];
    *(hashhex+15) = charset[(unsigned char)*(hash+7)&15];
    *(hashhex+14) = charset[(unsigned char)*(hash+7)>>4];
    *(hashhex+17) = charset[(unsigned char)*(hash+8)&15];
    *(hashhex+16) = charset[(unsigned char)*(hash+8)>>4];
    *(hashhex+19) = charset[(unsigned char)*(hash+9)&15];
    *(hashhex+18) = charset[(unsigned char)*(hash+9)>>4];
    *(hashhex+21) = charset[(unsigned char)*(hash+10)&15];
    *(hashhex+20) = charset[(unsigned char)*(hash+10)>>4];
    *(hashhex+23) = charset[(unsigned char)*(hash+11)&15];
    *(hashhex+22) = charset[(unsigned char)*(hash+11)>>4];
    *(hashhex+25) = charset[(unsigned char)*(hash+12)&15];
    *(hashhex+24) = charset[(unsigned char)*(hash+12)>>4];
    *(hashhex+27) = charset[(unsigned char)*(hash+13)&15];
    *(hashhex+26) = charset[(unsigned char)*(hash+13)>>4];
    *(hashhex+29) = charset[(unsigned char)*(hash+14)&15];
    *(hashhex+28) = charset[(unsigned char)*(hash+14)>>4];
    *(hashhex+31) = charset[(unsigned char)*(hash+15)&15];
    *(hashhex+30) = charset[(unsigned char)*(hash+15)>>4];
    *(hashhex+32) = 0;
}


/* SHA1 proto function */
void hash_proto_sha1(const char *plaintext, char *hash, int len)
{
    SHA_CTX ctx;
    SHA1_Init(&ctx);
    SHA1_Update(&ctx, plaintext, len);
    SHA1_Final((unsigned char *)hash,&ctx);
}


/* RIPEMD160 proto function */
void hash_proto_ripemd160(const char *plaintext, char *hash, int len)
{
    RIPEMD160_CTX ctx;
    RIPEMD160_Init(&ctx);
    RIPEMD160_Update(&ctx, plaintext, len);
    RIPEMD160_Final((unsigned char *)hash,&ctx);
}


/* SHA1 digest -> hex str */
void hash_proto_sha1_hex(const char *hash, char *hashhex)
{
    unsigned char charset[17]="0123456789abcdef";

    *(hashhex+1) = charset[(unsigned char)*(hash)&15];
    *(hashhex+0) = charset[(unsigned char)*(hash)>>4];
    *(hashhex+3) = charset[(unsigned char)*(hash+1)&15];
    *(hashhex+2) = charset[(unsigned char)*(hash+1)>>4];
    *(hashhex+5) = charset[(unsigned char)*(hash+2)&15];
    *(hashhex+4) = charset[(unsigned char)*(hash+2)>>4];
    *(hashhex+7) = charset[(unsigned char)*(hash+3)&15];
    *(hashhex+6) = charset[(unsigned char)*(hash+3)>>4];
    *(hashhex+9) = charset[(unsigned char)*(hash+4)&15];
    *(hashhex+8) = charset[(unsigned char)*(hash+4)>>4];
    *(hashhex+11) = charset[(unsigned char)*(hash+5)&15];
    *(hashhex+10) = charset[(unsigned char)*(hash+5)>>4];
    *(hashhex+13) = charset[(unsigned char)*(hash+6)&15];
    *(hashhex+12) = charset[(unsigned char)*(hash+6)>>4];
    *(hashhex+15) = charset[(unsigned char)*(hash+7)&15];
    *(hashhex+14) = charset[(unsigned char)*(hash+7)>>4];
    *(hashhex+17) = charset[(unsigned char)*(hash+8)&15];
    *(hashhex+16) = charset[(unsigned char)*(hash+8)>>4];
    *(hashhex+19) = charset[(unsigned char)*(hash+9)&15];
    *(hashhex+18) = charset[(unsigned char)*(hash+9)>>4];
    *(hashhex+21) = charset[(unsigned char)*(hash+10)&15];
    *(hashhex+20) = charset[(unsigned char)*(hash+10)>>4];
    *(hashhex+23) = charset[(unsigned char)*(hash+11)&15];
    *(hashhex+22) = charset[(unsigned char)*(hash+11)>>4];
    *(hashhex+25) = charset[(unsigned char)*(hash+12)&15];
    *(hashhex+24) = charset[(unsigned char)*(hash+12)>>4];
    *(hashhex+27) = charset[(unsigned char)*(hash+13)&15];
    *(hashhex+26) = charset[(unsigned char)*(hash+13)>>4];
    *(hashhex+29) = charset[(unsigned char)*(hash+14)&15];
    *(hashhex+28) = charset[(unsigned char)*(hash+14)>>4];
    *(hashhex+31) = charset[(unsigned char)*(hash+15)&15];
    *(hashhex+30) = charset[(unsigned char)*(hash+15)>>4];
    *(hashhex+33) = charset[(unsigned char)*(hash+16)&15];
    *(hashhex+32) = charset[(unsigned char)*(hash+16)>>4];
    *(hashhex+35) = charset[(unsigned char)*(hash+17)&15];
    *(hashhex+34) = charset[(unsigned char)*(hash+17)>>4];
    *(hashhex+37) = charset[(unsigned char)*(hash+18)&15];
    *(hashhex+36) = charset[(unsigned char)*(hash+18)>>4];
    *(hashhex+39) = charset[(unsigned char)*(hash+19)&15];
    *(hashhex+38) = charset[(unsigned char)*(hash+19)>>4];
    *(hashhex+40) = 0;
}


/* SHA256 proto function */
void hash_proto_sha256(const char *plaintext, char *hash, int len)
{
    SHA256_CTX ctx;
    SHA256_Init(&ctx);
    SHA256_Update(&ctx, plaintext, len);
    SHA256_Final((unsigned char *)hash,&ctx);
}


/* SHA256 -> hexstr */
void hash_proto_sha256_hex(const char *hash, char *hashhex)
{
    unsigned char charset[17]="0123456789abcdef";

    *(hashhex+1) = charset[(unsigned char)*(hash)&15];
    *(hashhex+0) = charset[(unsigned char)*(hash)>>4];
    *(hashhex+3) = charset[(unsigned char)*(hash+1)&15];
    *(hashhex+2) = charset[(unsigned char)*(hash+1)>>4];
    *(hashhex+5) = charset[(unsigned char)*(hash+2)&15];
    *(hashhex+4) = charset[(unsigned char)*(hash+2)>>4];
    *(hashhex+7) = charset[(unsigned char)*(hash+3)&15];
    *(hashhex+6) = charset[(unsigned char)*(hash+3)>>4];
    *(hashhex+9) = charset[(unsigned char)*(hash+4)&15];
    *(hashhex+8) = charset[(unsigned char)*(hash+4)>>4];
    *(hashhex+11) = charset[(unsigned char)*(hash+5)&15];
    *(hashhex+10) = charset[(unsigned char)*(hash+5)>>4];
    *(hashhex+13) = charset[(unsigned char)*(hash+6)&15];
    *(hashhex+12) = charset[(unsigned char)*(hash+6)>>4];
    *(hashhex+15) = charset[(unsigned char)*(hash+7)&15];
    *(hashhex+14) = charset[(unsigned char)*(hash+7)>>4];
    *(hashhex+17) = charset[(unsigned char)*(hash+8)&15];
    *(hashhex+16) = charset[(unsigned char)*(hash+8)>>4];
    *(hashhex+19) = charset[(unsigned char)*(hash+9)&15];
    *(hashhex+18) = charset[(unsigned char)*(hash+9)>>4];
    *(hashhex+21) = charset[(unsigned char)*(hash+10)&15];
    *(hashhex+20) = charset[(unsigned char)*(hash+10)>>4];
    *(hashhex+23) = charset[(unsigned char)*(hash+11)&15];
    *(hashhex+22) = charset[(unsigned char)*(hash+11)>>4];
    *(hashhex+25) = charset[(unsigned char)*(hash+12)&15];
    *(hashhex+24) = charset[(unsigned char)*(hash+12)>>4];
    *(hashhex+27) = charset[(unsigned char)*(hash+13)&15];
    *(hashhex+26) = charset[(unsigned char)*(hash+13)>>4];
    *(hashhex+29) = charset[(unsigned char)*(hash+14)&15];
    *(hashhex+28) = charset[(unsigned char)*(hash+14)>>4];
    *(hashhex+31) = charset[(unsigned char)*(hash+15)&15];
    *(hashhex+30) = charset[(unsigned char)*(hash+15)>>4];
    *(hashhex+33) = charset[(unsigned char)*(hash+16)&15];
    *(hashhex+32) = charset[(unsigned char)*(hash+16)>>4];
    *(hashhex+35) = charset[(unsigned char)*(hash+17)&15];
    *(hashhex+34) = charset[(unsigned char)*(hash+17)>>4];
    *(hashhex+37) = charset[(unsigned char)*(hash+18)&15];
    *(hashhex+36) = charset[(unsigned char)*(hash+18)>>4];
    *(hashhex+39) = charset[(unsigned char)*(hash+19)&15];
    *(hashhex+38) = charset[(unsigned char)*(hash+19)>>4];
    *(hashhex+40) = charset[(unsigned char)*(hash+20)>>4];
    *(hashhex+41) = charset[(unsigned char)*(hash+20)&15];
    *(hashhex+42) = charset[(unsigned char)*(hash+21)>>4];
    *(hashhex+43) = charset[(unsigned char)*(hash+21)&15];
    *(hashhex+44) = charset[(unsigned char)*(hash+22)>>4];
    *(hashhex+45) = charset[(unsigned char)*(hash+22)&15];
    *(hashhex+46) = charset[(unsigned char)*(hash+23)>>4];
    *(hashhex+47) = charset[(unsigned char)*(hash+23)&15];
    *(hashhex+48) = charset[(unsigned char)*(hash+24)>>4];
    *(hashhex+49) = charset[(unsigned char)*(hash+24)&15];
    *(hashhex+50) = charset[(unsigned char)*(hash+25)>>4];
    *(hashhex+51) = charset[(unsigned char)*(hash+25)&15];
    *(hashhex+52) = charset[(unsigned char)*(hash+26)>>4];
    *(hashhex+53) = charset[(unsigned char)*(hash+26)&15];
    *(hashhex+54) = charset[(unsigned char)*(hash+27)>>4];
    *(hashhex+55) = charset[(unsigned char)*(hash+27)&15];
    *(hashhex+56) = charset[(unsigned char)*(hash+28)>>4];
    *(hashhex+57) = charset[(unsigned char)*(hash+28)&15];
    *(hashhex+58) = charset[(unsigned char)*(hash+29)>>4];
    *(hashhex+59) = charset[(unsigned char)*(hash+29)&15];
    *(hashhex+60) = charset[(unsigned char)*(hash+30)>>4];
    *(hashhex+61) = charset[(unsigned char)*(hash+30)&15];
    *(hashhex+62) = charset[(unsigned char)*(hash+31)>>4];
    *(hashhex+63) = charset[(unsigned char)*(hash+31)&15];
    *(hashhex+64) = 0;
}


/* proto SHA512 function */
void hash_proto_sha512(const char *plaintext, char *hash, int len)
{
    SHA512_CTX ctx;
    SHA512_Init(&ctx);
    SHA512_Update(&ctx, plaintext, len);
    SHA512_Final((unsigned char *)hash,&ctx);
}



/* SHA512 -> hexstr */
void hash_proto_sha512_hex(const char *hash, char *hashhex)
{
    unsigned char charset[17]="0123456789abcdef";

    *(hashhex+1) = charset[(unsigned char)*(hash)&15];
    *(hashhex+0) = charset[(unsigned char)*(hash)>>4];
    *(hashhex+3) = charset[(unsigned char)*(hash+1)&15];
    *(hashhex+2) = charset[(unsigned char)*(hash+1)>>4];
    *(hashhex+5) = charset[(unsigned char)*(hash+2)&15];
    *(hashhex+4) = charset[(unsigned char)*(hash+2)>>4];
    *(hashhex+7) = charset[(unsigned char)*(hash+3)&15];
    *(hashhex+6) = charset[(unsigned char)*(hash+3)>>4];
    *(hashhex+9) = charset[(unsigned char)*(hash+4)&15];
    *(hashhex+8) = charset[(unsigned char)*(hash+4)>>4];
    *(hashhex+11) = charset[(unsigned char)*(hash+5)&15];
    *(hashhex+10) = charset[(unsigned char)*(hash+5)>>4];
    *(hashhex+13) = charset[(unsigned char)*(hash+6)&15];
    *(hashhex+12) = charset[(unsigned char)*(hash+6)>>4];
    *(hashhex+15) = charset[(unsigned char)*(hash+7)&15];
    *(hashhex+14) = charset[(unsigned char)*(hash+7)>>4];
    *(hashhex+17) = charset[(unsigned char)*(hash+8)&15];
    *(hashhex+16) = charset[(unsigned char)*(hash+8)>>4];
    *(hashhex+19) = charset[(unsigned char)*(hash+9)&15];
    *(hashhex+18) = charset[(unsigned char)*(hash+9)>>4];
    *(hashhex+21) = charset[(unsigned char)*(hash+10)&15];
    *(hashhex+20) = charset[(unsigned char)*(hash+10)>>4];
    *(hashhex+23) = charset[(unsigned char)*(hash+11)&15];
    *(hashhex+22) = charset[(unsigned char)*(hash+11)>>4];
    *(hashhex+25) = charset[(unsigned char)*(hash+12)&15];
    *(hashhex+24) = charset[(unsigned char)*(hash+12)>>4];
    *(hashhex+27) = charset[(unsigned char)*(hash+13)&15];
    *(hashhex+26) = charset[(unsigned char)*(hash+13)>>4];
    *(hashhex+29) = charset[(unsigned char)*(hash+14)&15];
    *(hashhex+28) = charset[(unsigned char)*(hash+14)>>4];
    *(hashhex+31) = charset[(unsigned char)*(hash+15)&15];
    *(hashhex+30) = charset[(unsigned char)*(hash+15)>>4];
    *(hashhex+33) = charset[(unsigned char)*(hash+16)&15];
    *(hashhex+32) = charset[(unsigned char)*(hash+16)>>4];
    *(hashhex+35) = charset[(unsigned char)*(hash+17)&15];
    *(hashhex+34) = charset[(unsigned char)*(hash+17)>>4];
    *(hashhex+37) = charset[(unsigned char)*(hash+18)&15];
    *(hashhex+36) = charset[(unsigned char)*(hash+18)>>4];
    *(hashhex+39) = charset[(unsigned char)*(hash+19)&15];
    *(hashhex+38) = charset[(unsigned char)*(hash+19)>>4];
    *(hashhex+40) = charset[(unsigned char)*(hash+20)>>4];
    *(hashhex+41) = charset[(unsigned char)*(hash+20)&15];
    *(hashhex+42) = charset[(unsigned char)*(hash+21)>>4];
    *(hashhex+43) = charset[(unsigned char)*(hash+21)&15];
    *(hashhex+44) = charset[(unsigned char)*(hash+22)>>4];
    *(hashhex+45) = charset[(unsigned char)*(hash+22)&15];
    *(hashhex+46) = charset[(unsigned char)*(hash+23)>>4];
    *(hashhex+47) = charset[(unsigned char)*(hash+23)&15];
    *(hashhex+48) = charset[(unsigned char)*(hash+24)>>4];
    *(hashhex+49) = charset[(unsigned char)*(hash+24)&15];
    *(hashhex+50) = charset[(unsigned char)*(hash+25)>>4];
    *(hashhex+51) = charset[(unsigned char)*(hash+25)&15];
    *(hashhex+52) = charset[(unsigned char)*(hash+26)>>4];
    *(hashhex+53) = charset[(unsigned char)*(hash+26)&15];
    *(hashhex+54) = charset[(unsigned char)*(hash+27)>>4];
    *(hashhex+55) = charset[(unsigned char)*(hash+27)&15];
    *(hashhex+56) = charset[(unsigned char)*(hash+28)>>4];
    *(hashhex+57) = charset[(unsigned char)*(hash+28)&15];
    *(hashhex+58) = charset[(unsigned char)*(hash+29)>>4];
    *(hashhex+59) = charset[(unsigned char)*(hash+29)&15];
    *(hashhex+60) = charset[(unsigned char)*(hash+30)>>4];
    *(hashhex+61) = charset[(unsigned char)*(hash+30)&15];
    *(hashhex+62) = charset[(unsigned char)*(hash+31)>>4];
    *(hashhex+63) = charset[(unsigned char)*(hash+31)&15];
    *(hashhex+64) = charset[(unsigned char)*(hash+32)>>4];
    *(hashhex+65) = charset[(unsigned char)*(hash+32)&15];
    *(hashhex+66) = charset[(unsigned char)*(hash+33)>>4];
    *(hashhex+67) = charset[(unsigned char)*(hash+33)&15];
    *(hashhex+68) = charset[(unsigned char)*(hash+34)>>4];
    *(hashhex+69) = charset[(unsigned char)*(hash+34)&15];
    *(hashhex+70) = charset[(unsigned char)*(hash+35)>>4];
    *(hashhex+71) = charset[(unsigned char)*(hash+35)&15];
    *(hashhex+72) = charset[(unsigned char)*(hash+36)>>4];
    *(hashhex+73) = charset[(unsigned char)*(hash+36)&15];
    *(hashhex+74) = charset[(unsigned char)*(hash+37)>>4];
    *(hashhex+75) = charset[(unsigned char)*(hash+37)&15];
    *(hashhex+76) = charset[(unsigned char)*(hash+38)>>4];
    *(hashhex+77) = charset[(unsigned char)*(hash+38)&15];
    *(hashhex+78) = charset[(unsigned char)*(hash+39)>>4];
    *(hashhex+79) = charset[(unsigned char)*(hash+39)&15];
    *(hashhex+80) = charset[(unsigned char)*(hash+40)>>4];
    *(hashhex+81) = charset[(unsigned char)*(hash+40)&15];
    *(hashhex+82) = charset[(unsigned char)*(hash+41)>>4];
    *(hashhex+83) = charset[(unsigned char)*(hash+41)&15];
    *(hashhex+84) = charset[(unsigned char)*(hash+42)>>4];
    *(hashhex+85) = charset[(unsigned char)*(hash+42)&15];
    *(hashhex+86) = charset[(unsigned char)*(hash+43)>>4];
    *(hashhex+87) = charset[(unsigned char)*(hash+43)&15];
    *(hashhex+88) = charset[(unsigned char)*(hash+44)>>4];
    *(hashhex+89) = charset[(unsigned char)*(hash+44)&15];
    *(hashhex+90) = charset[(unsigned char)*(hash+45)>>4];
    *(hashhex+91) = charset[(unsigned char)*(hash+45)&15];
    *(hashhex+92) = charset[(unsigned char)*(hash+46)>>4];
    *(hashhex+93) = charset[(unsigned char)*(hash+46)&15];
    *(hashhex+94) = charset[(unsigned char)*(hash+47)>>4];
    *(hashhex+95) = charset[(unsigned char)*(hash+47)&15];
    *(hashhex+96) = charset[(unsigned char)*(hash+48)>>4];
    *(hashhex+97) = charset[(unsigned char)*(hash+48)&15];
    *(hashhex+98) = charset[(unsigned char)*(hash+49)>>4];
    *(hashhex+99) = charset[(unsigned char)*(hash+49)&15];
    *(hashhex+100) = charset[(unsigned char)*(hash+50)>>4];
    *(hashhex+101) = charset[(unsigned char)*(hash+50)&15];
    *(hashhex+102) = charset[(unsigned char)*(hash+51)>>4];
    *(hashhex+103) = charset[(unsigned char)*(hash+51)&15];
    *(hashhex+104) = charset[(unsigned char)*(hash+52)>>4];
    *(hashhex+105) = charset[(unsigned char)*(hash+52)&15];
    *(hashhex+106) = charset[(unsigned char)*(hash+53)>>4];
    *(hashhex+107) = charset[(unsigned char)*(hash+53)&15];
    *(hashhex+108) = charset[(unsigned char)*(hash+54)>>4];
    *(hashhex+109) = charset[(unsigned char)*(hash+54)&15];
    *(hashhex+110) = charset[(unsigned char)*(hash+55)>>4];
    *(hashhex+111) = charset[(unsigned char)*(hash+55)&15];
    *(hashhex+112) = charset[(unsigned char)*(hash+56)>>4];
    *(hashhex+113) = charset[(unsigned char)*(hash+56)&15];
    *(hashhex+114) = charset[(unsigned char)*(hash+57)>>4];
    *(hashhex+115) = charset[(unsigned char)*(hash+57)&15];
    *(hashhex+116) = charset[(unsigned char)*(hash+58)>>4];
    *(hashhex+117) = charset[(unsigned char)*(hash+58)&15];
    *(hashhex+118) = charset[(unsigned char)*(hash+59)>>4];
    *(hashhex+119) = charset[(unsigned char)*(hash+59)&15];
    *(hashhex+120) = charset[(unsigned char)*(hash+60)>>4];
    *(hashhex+121) = charset[(unsigned char)*(hash+60)&15];
    *(hashhex+122) = charset[(unsigned char)*(hash+61)>>4];
    *(hashhex+123) = charset[(unsigned char)*(hash+61)&15];
    *(hashhex+124) = charset[(unsigned char)*(hash+62)>>4];
    *(hashhex+125) = charset[(unsigned char)*(hash+62)&15];
    *(hashhex+126) = charset[(unsigned char)*(hash+63)>>4];
    *(hashhex+127) = charset[(unsigned char)*(hash+63)&15];
    *(hashhex+128) = 0;
}


/* DES_fcrypt() from openssl */
void hash_proto_fcrypt(const char *password, const char *salt, char *ret)
{
    DES_fcrypt(password, salt, ret);
} 



/* BIO_s_mem -> readonly BIO with contents taken from FILE *file */
void hash_proto_new_biomem(FILE *file)
{
    char filedata[8192];

    fread(filedata, 8192, 1, file);
    filedata[8191] = 0;
    memcpy(privkey,filedata,8192);
}



/* PEM_read_RSAPrivateKey from openssl */
void hash_proto_PEM_readfile(const char *passphrase, int *RSAret)
{
    EVP_PKEY *mykey;
    char asan[50];
    BIO *localbio;

    localbio = BIO_new_mem_buf(privkey, -1);
    mykey = PEM_read_bio_PrivateKey(localbio, NULL, 0, (void *)passphrase);
    BIO_free(localbio);
    if (mykey!=NULL) {*RSAret = 1;}

}


/* PBKDF2 from OpenSSL */
void hash_proto_pbkdf2(const char *pass, unsigned char *salt, int saltlen, int iter, int keylen, unsigned char *out)
{
    (void)PKCS5_PBKDF2_HMAC_SHA1(pass, strlen(pass), salt, saltlen, iter, keylen, out);
}



/* PBKDF2 with HMAC_SHA512 */
void hash_proto_pbkdf512(const char *pass, unsigned char *salt, int saltlen, int iter, int keylen, unsigned char *out)
{
    unsigned char digtmp[SHA512_DIGEST_LENGTH], *p, itmp[4];
    int cplen, j, k, tkeylen;
    unsigned long i = 1;
    HMAC_CTX hctx;
    int passlen = strlen(pass);

    HMAC_CTX_init(&hctx);
    p = out;
    tkeylen = keylen;
    if(!pass) passlen = 0;
    else if(passlen == -1) passlen = strlen(pass);
    while(tkeylen) 
    {
        if(tkeylen > SHA512_DIGEST_LENGTH) cplen = SHA512_DIGEST_LENGTH;
        else cplen = tkeylen;
        itmp[0] = (unsigned char)((i >> 24) & 0xff);
        itmp[1] = (unsigned char)((i >> 16) & 0xff);
        itmp[2] = (unsigned char)((i >> 8) & 0xff);
        itmp[3] = (unsigned char)(i & 0xff);
        HMAC_Init_ex(&hctx, pass, passlen, EVP_sha512(), NULL);
        HMAC_Update(&hctx, salt, saltlen);
        HMAC_Update(&hctx, itmp, 4);
        HMAC_Final(&hctx, digtmp, NULL);
        memcpy(p, digtmp, cplen);
        for(j = 1; j < iter; j++) 
        {
    	    HMAC(EVP_sha512(), pass, passlen, digtmp, SHA_DIGEST_LENGTH, digtmp, NULL);
    	    for(k = 0; k < cplen; k++) p[k] ^= digtmp[k];
	}
	tkeylen-= cplen;
	i++;
	p+= cplen;
    }
    HMAC_CTX_cleanup(&hctx);
}



/* HMAC_SHA1 from OpenSSL on files */
void hash_proto_hmac_sha1_file(void *key, int keylen, char *filename, long offset, long size, unsigned char *output, int outputlen)
{
    int fd;
    char buf[4096];
    long localoff;
    int bufread;
    HMAC_CTX ctx;
    
    HMAC_CTX_init(&ctx);
    HMAC_Init(&ctx, key, keylen, EVP_sha1());
    fd = open(filename, O_RDONLY);
    if (fd<1)
    {
	memcpy(output,key,outputlen);
	return;
    }
    if (lseek(fd, offset, SEEK_SET) == -1)
    {
	memcpy(output,key,outputlen);
	return;
    }
    localoff = offset;

    while (localoff < (offset+size))
    {
	if ((localoff+4096) > (offset+size)) 
	{
	    bufread = (offset+size) - localoff;
	    localoff = offset + size;
	}
	else
	{
	    bufread = 4096;
	    localoff += 4096;
	}
	read(fd, buf, bufread);
	HMAC_Update(&ctx, buf, bufread);
    }

    HMAC_Final(&ctx, output, &outputlen);
    close(fd);
    HMAC_CTX_cleanup(&ctx);
}


/* AES encrypt openssl wrapper */
/* mode: use 0 for ECB and 1 for CBC */
void hash_proto_aes_encrypt(const unsigned char *key, int keysize, const unsigned char *in, int len, unsigned char *vec, unsigned char *out, int mode)
{
    AES_KEY akey;
    AES_set_encrypt_key(key, keysize*8, &akey);
    switch (mode)
    {
	case 0:
	    AES_ecb_encrypt(in, out, &akey, AES_ENCRYPT);
	    break;
	case 1:
	    AES_cbc_encrypt(in, out, len, &akey, vec, AES_ENCRYPT);
	    break;
	default: return;
    }
}

/* AES decrypt openssl wrapper */
/* mode: use 0 for ECB and 1 for CBC */
void hash_proto_aes_decrypt(const unsigned char *key, int keysize, const unsigned char *in, int len, unsigned char *vec, unsigned char *out, int mode)
{
    AES_KEY akey;
    AES_set_decrypt_key(key, keysize*8, &akey);
    switch (mode)
    {
	case 0:
	    AES_ecb_encrypt(in, out, &akey, AES_DECRYPT);
	    break;
	case 1:
	    AES_cbc_encrypt(in, out, len, &akey, vec, AES_DECRYPT);
	    break;
	default: return;
    }
}




/* Add username to list temp */
void hash_proto_add_username(const char *username)
{
    if (username) memcpy((char *)&temp_username, username, strlen(username));
    else temp_username[0]=0;
}


/* Add hash to list temp */
void hash_proto_add_hash(const char *hash, int len)
{
    if (hash)
    {
	if (len==0)  {memcpy((char *)&temp_hash, hash, strlen(hash));hash_is_raw = 0;}
	if (len !=0) {memcpy((char *)&temp_hash, hash, len);hash_is_raw = 1;}
	if (len!=0) hash_ret_len = len;
	else hash_ret_len = strlen(hash);
    }
    else temp_hash[0]=0;
}


/* Add salt to list temp */
void hash_proto_add_salt(const char *salt)
{
    if (salt) memcpy((char *)&temp_salt, salt, strlen(salt));
    else temp_salt[0]=0;
}


/* Add salt2 to list temp */
void hash_proto_add_salt2(const char *salt2)
{
    if (salt2) memcpy((char *)&temp_salt2, salt2, strlen(salt2));
    else temp_salt2[0]=0;
}



/* Add to hash list */
hash_stat add_hash_list(char *username, char *hash, char *salt, char *salt2)
{
    struct hash_list_s *temp_list, *temp_list1;
    
    temp_list = hash_list;
    if (temp_list) 
    {
	temp_list1 = temp_list;
	while (temp_list1->next) temp_list1 = temp_list1->next;
	posix_memalign((void **)&temp_list,64,sizeof(struct hash_list_s));
	temp_list->next = NULL;
	temp_list->prev = temp_list1;
	temp_list1->next =  temp_list;
    }
    /* First entry? */
    else 
    {
	temp_list = malloc(sizeof(struct hash_list_s));
	hash_list = temp_list;
	hash_list->next = NULL;
	hash_list->prev = NULL;
    }
    
    if (!temp_list)
    {
	elog("Not enough memory for new hash_list entry! %s\n","");
	return hash_err;
    }
    
    if (username)
    {
	temp_list->username = malloc(HASHFILE_MAX_LINE_LENGTH);
	strcpy(temp_list->username, username);
    }
    if (hash)
    {
	temp_list->hash = malloc(HASHFILE_MAX_LINE_LENGTH);
	memcpy(temp_list->hash, hash, hash_ret_len);
    }
    if (salt)
    {
	temp_list->salt = malloc(HASHFILE_MAX_LINE_LENGTH);
	strcpy(temp_list->salt, salt);
    }
    if (salt2)
    {
	temp_list->salt2 = malloc(HASHFILE_MAX_LINE_LENGTH);
	strcpy(temp_list->salt2, salt2);
    }
    
    return hash_ok;
}



/* Add to cracked list */
hash_stat add_cracked_list(char *username, char *hash, char *salt, char *salt2)
{
    struct hash_list_s *temp_list, *temp_list1;
    
    temp_list = cracked_list;
    if (temp_list) 
    {
	temp_list1 = temp_list;
	while (temp_list1->next) temp_list1 = temp_list1->next;
	temp_list = malloc(sizeof(struct hash_list_s));
	temp_list->next = NULL;
	temp_list->prev = temp_list1;
	temp_list1->next =  temp_list;
    }
    /* First entry? */
    else 
    {
	temp_list = malloc(sizeof(struct hash_list_s));
	cracked_list = temp_list;
	cracked_list->next = NULL;
	cracked_list->prev = NULL;
    }
    
    if (!temp_list)
    {
	elog("Not enough memory for new crack_list entry! %s\n","");
	return hash_err;
    }
    
    if (username)
    {
	temp_list->username = malloc(HASHFILE_MAX_LINE_LENGTH+1);
	strcpy(temp_list->username, username);
    }
    if (hash)
    {
	temp_list->hash = malloc(HASHFILE_MAX_LINE_LENGTH+1);
	memcpy(temp_list->hash, hash, hash_ret_len);
    }
    if (salt)
    {
	temp_list->salt = malloc(HASHFILE_MAX_LINE_LENGTH+1);
	strcpy(temp_list->salt, salt);
    }
    if (salt2)
    {
	temp_list->salt2 = malloc(HASHFILE_MAX_LINE_LENGTH+1);
	strcpy(temp_list->salt2, salt2);
    }
    
    return hash_ok;
}


/* Delete hash list node */
hash_stat del_hash_list(struct hash_list_s *node)
{
    struct hash_list_s *temp_list;
    int first=0;

    if (!node) return hash_err;
    pthread_mutex_lock(&listmutex);
    if ((node->prev == NULL) && (node->next == NULL)) first = 1;
    temp_list = node->next;
    if (temp_list) temp_list->prev = node->prev;
    temp_list = node->prev;
    if (temp_list) temp_list->next = node->next;
    if (node->username) free(node->username);
    if (node->hash) free(node->hash);
    if (node->salt) free(node->salt);
    if (node->salt2) free(node->salt2);
    node->username = node->hash = node-> salt = node->salt2 = NULL;
    node = NULL;
    if (first == 1) hash_list = NULL;
    pthread_mutex_unlock(&listmutex);
    return hash_ok;
}


/* Delete cracked list node */
hash_stat del_cracked_list(struct hash_list_s *node)
{
    struct hash_list_s *temp_list;

    if (!node) return hash_err;
    pthread_mutex_lock(&crackedmutex);
    temp_list = node->next;
    if (temp_list) temp_list->prev = node->prev;
    temp_list = node->prev;
    if (temp_list) temp_list->next = node->next;
    free(node->username);
    free(node->hash);
    free(node->salt);
    free(node->salt2);
    free(node);
    if (node==cracked_list) 
    {
	node = temp_list;
    }
    else node = NULL;
    pthread_mutex_unlock(&crackedmutex);
    return hash_ok;
}


static void str2hex(char *str, char *hex, int size)
{
    int cnt;
    char charset[16]="0123456789abcdef";
    unsigned char t1, t2;
    
    for (cnt=0;cnt<size;cnt++)
    {

	*(hex+(2*cnt)+1) = charset[(unsigned char)*(str+cnt)&15];
	*(hex+(2*cnt)) = charset[(unsigned char)*(str+cnt)>>4];
    }
    *(hex+(2*size)) = 0;
    *(hex+(2*size)+1) = 0;

}


/* Print the hashes linked list */
void print_hash_list(void)
{
    struct hash_list_s *temp_list = hash_list;
    char hashhex[HASHFILE_MAX_LINE_LENGTH];
    
    hash_is_raw = hash_plugin_is_raw();
    
    printf("...Hash list...\n\n");
    if (!temp_list) printf("empty\n");
    while (temp_list) 
    {
	if (temp_list->username) printf("Username: %s ", temp_list->username);

	if (hash_is_raw==1)
	{
	    if (temp_list->hash) 
	    {
		str2hex(temp_list->hash, hashhex, hash_ret_len);
		printf(" Hash: %s ", hashhex);
	    }
	}
	else if (temp_list->hash) printf(" Hash: %s ", temp_list->hash);
	    
	if (temp_list->salt) printf(" Salt: %s ", temp_list->salt);
	if (temp_list->salt2) printf(" Salt2: %s ", temp_list->salt2);
	printf("\n");
	temp_list=temp_list->next;
    }
}



/* Print cracked hashes linked list */
void print_cracked_list(void)
{
    struct hash_list_s *temp_list = cracked_list;
    void *zero;
    char outpline[HASHFILE_MAX_LINE_LENGTH];
    char hashhex[HASHFILE_MAX_LINE_LENGTH];

    
    printf("\n");
    hlog("-= Cracked list =-\n%s\n","");
    printf("Username: \t\tHash: \t\t\t\t\t\tPreimage:\n");
    printf("-----------------------------------------------------------------------------------\n");

    while (temp_list)
    {

	while (strlen(temp_list->username)<20) strcat(temp_list->username," ");
	while (strlen(temp_list->salt)<20) strcat(temp_list->salt," ");
	snprintf(outpline,16,"%s             ", temp_list->username);
	printf("%s \t",outpline);
	
	
	if (hash_is_raw == 1)
	{
	    if (temp_list->hash) 
	    {
		str2hex(temp_list->hash, (char *)&hashhex, hash_ret_len);
		snprintf(outpline, 41, "%s                        ", hashhex);
	    }
	}
	else if (temp_list->hash) snprintf(outpline,41,"%s                            ", temp_list->hash);
	
	printf("%s \t",outpline);
	snprintf(outpline,30,"%s             ", temp_list->salt2);
	printf("%s\n",outpline);
	temp_list = temp_list->next;
    }
    printf("\n");
}


/* Print cracked hashes linked list */
void print_cracked_list_to_file(char *filename)
{
    struct hash_list_s *temp_list = cracked_list;
    void *zero;
    char outpline[HASHFILE_MAX_LINE_LENGTH];
    char hashhex[HASHFILE_MAX_LINE_LENGTH];
    FILE *fp;
    int fd;
    struct stat filest;
    
    

    fd = open(filename, O_WRONLY | O_NOFOLLOW | O_CREAT | O_TRUNC, 0640);
    fp = fdopen(fd, "w");
    if (fstat(fd, &filest) != 0)
    {
        printf("\n");
        elog("Cannot stat output file: %s - probably a symlink attack!!!\n", filename);
        return ;
    }
    if (filest.st_nlink > 1)
    {
        printf("\n");
        elog("Hardlink attack attempted! Output file: %s Please remove the hardlink manually!\n", filename);
        fclose(fp);
        return;
    }
    
    
    fprintf(fp,"\n");
    fprintf(fp, "Username: \tHash: \t\t\t\t\t\tPreimage:\n");
    fprintf(fp, "-------------------------------------------------------------------------\n");

    while (temp_list)
    {

	while (strlen(temp_list->username)<20) strcat(temp_list->username," ");
	while (strlen(temp_list->salt)<20) strcat(temp_list->salt," ");
	snprintf(outpline,10,"%s             ", temp_list->username);
	fprintf(fp, "%s \t",outpline);
	
	
	if (hash_is_raw==1)
	{
	    if (temp_list->hash) 
	    {
		str2hex(temp_list->hash, (char *)&hashhex, hash_ret_len);
		snprintf(outpline, 41, " %s                        ", hashhex);
	    }
	}
	else if (temp_list->hash) snprintf(outpline,41,"%s                            ", temp_list->hash);
	
	fprintf(fp, "%s \t",outpline);
	snprintf(outpline, 30, "%s             ", temp_list->salt2);
	fprintf(fp, "%s\n",outpline);
	temp_list = temp_list->next;
    }
    fprintf(fp, "\n");
}





/* Get number of cracked hashes */
int get_cracked_num(void)
{
    struct hash_list_s *temp_list = cracked_list;
    int num = 0;
    
    while (temp_list)
    {
	num++;
	temp_list = temp_list->next;
    }
    return num;
}


/* Get number of hashes in list */
int get_hashes_num(void)
{
    struct hash_list_s *temp_list = hash_list;
    int num = 0;
    
    while (temp_list)
    {
	num++;
	temp_list = temp_list->next;
    }
    return num;
}


/* Free memory used by lists */
void cleanup_lists(void)
{
    struct hash_list_s *temp_list;


    temp_list = hash_list;
    while (temp_list)
    {
	if (temp_list->next) del_hash_list(temp_list->next);
	temp_list = hash_list->next;
    }
    if (hash_list) del_hash_list(hash_list);
    temp_list = cracked_list;
    while (temp_list)
    {
	if (temp_list->next) del_cracked_list(temp_list->next);
	temp_list = cracked_list->next;
    }
    if (cracked_list) del_cracked_list(cracked_list);


}


/* Initialize Markov attack data */
void markov_attack_init(void)
{
    int a,b,c;
    
    for (a=0;a<88;a++) markov0[a]=0;
    for (a=0;a<88;a++) 
	for (b=0;b<88;b++) markov1[a][b]=0;
    markov_threshold = 0;
    markov_max_len = 8;
    bzero(markov_statfile,255);
    markov_charset = malloc(88);
    strcpy(markov_statfile,"english");
    strcpy(markov_charset,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789~!{}@#$%^&*()-+[]|\\;':,./");
}

/* Print out Markov statfiles summary */
void markov_print_statfiles(void)
{
    DIR *dir;
    FILE *statfile;
    struct dirent *dentry;
    char statname[1024];
    char readline[256];
    char readline2[256];
    int avg;
    int cnt=0;

    dir=opendir("/usr/share/hashkill/markov");
    if (!dir)
    {
        elog("Cannot open markov stats dir: %s", "/usr/share/hashkill/markov");
        return;
    }
    hlog("Markov attack statfiles list: %s\n\n","");
    printf("Statfile: \t\tDef.threshold\tShort description:\n"
	    "--------------------------------------------------------------------------------\n");
    do
    {
        errno = 0;
        if ((dentry = readdir(dir)) != NULL)
        {
            if ((dentry->d_type == DT_REG) && (strstr(dentry->d_name, ".stat")))
            {
                snprintf(statname,1024,"/usr/share/hashkill/markov/%s", dentry->d_name);
                /* Parse the first two lines from statfile */
                statfile = fopen(statname, "r");
                if (!statfile)
                {
                    goto next;
                }
                if (!fgets(readline, 256, statfile))
                {
                    elog("Cannot read from statfile : %s\n",statname);
                    goto next;
                }
                readline[strlen(readline)-1] = 0;
                if (!fgets(readline2, 256, statfile))
                {
                    elog("Cannot read from statfile : %s\n",statname);
                    goto next;
                }
                avg = atoi(readline2);
                snprintf(readline2, 20, "%s               ", strtok(dentry->d_name,"."));
                
                printf("\033[1;33m%s\033[1;0m\t%d\t\t%s\n", readline2, avg, readline);

            next:
            if (statfile) fclose(statfile);
            }
        }
    } while (dentry != NULL);
    closedir(dir);
    printf("\n");
}

/* Load Markov statfile */
hash_stat markov_load_statfile(char *statname)
{
    FILE *fd;
    char buf[HASHFILE_MAX_LINE_LENGTH];
    int cnt1,cnt2;
    char c1,c2;
    
    strcpy(markov_statfile, statname);
    sprintf(buf,"/usr/share/hashkill/markov/%s.stat",statname);
    fd = fopen(buf,"r");
    if (!fd)
    {
	elog("Cannot open Markov statfile: %s\n",buf);
	return hash_err;
    }
    fgets(buf, HASHFILE_MAX_LINE_LENGTH, fd);
    buf[strlen(buf)-1] = 0;
    hlog("Loading markov statfile (%s)\n", statname);
    fgets(buf, HASHFILE_MAX_LINE_LENGTH, fd);
    if (markov_threshold==0) markov_threshold = atoi(buf);
    /* TODO: LOAD TABLES! */
    for (cnt1=0;cnt1<88;cnt1++) fscanf(fd, "%c %d\n", &c1, &markov0[cnt1]);
    for (cnt1=0;cnt1<88;cnt1++) 
	for (cnt2=0;cnt2<88;cnt2++) 
	{
	    fscanf(fd, "%c %c %d\n", &c1, &c2, &markov1[cnt1][cnt2]);
	}

    
    fclose(fd);
    return hash_ok;
}
